<?php

namespace Drupal\nostr_simple_publish\Services;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Site\Settings;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Psr\Log\LoggerInterface;
use swentel\nostr\Sign;
use WebSocket\Client;

class NostrPublish implements NostrPublishInterface {

  use StringTranslationTrait;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected AccountInterface $currentUser;

  /**
   * The Messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected MessengerInterface $messenger;

  /**
   * Nostr storage service.
   *
   * @var \Drupal\nostr_simple_publish\Services\NostrStorageInterface
   */
  protected NostrStorageInterface $nostrStorage;

  /**
   * A logger instance.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected LoggerInterface $logger;

  /**
   * Show Drupal message or not.
   *
   * @var bool
   */
  protected bool $showDrupalMessage = FALSE;

  /**
   * NostrPublish constructor.
   *
   * @param \Drupal\Core\Session\AccountInterface $current_user
   *   The current user.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger.
   * @param \Psr\Log\LoggerInterface $logger
   *   The logger.
   * @param \Drupal\nostr_simple_publish\Services\NostrStorageInterface $nostr_storage
   *   The nostr storage.
   */
  public function __construct(AccountInterface $current_user, MessengerInterface $messenger, LoggerInterface $logger, NostrStorageInterface $nostr_storage) {
    $this->currentUser = $current_user;
    $this->nostrStorage = $nostr_storage;
    $this->messenger = $messenger;
    $this->logger = $logger;
  }

  /**
   * {@inheritdoc}
   */
  public function sendMessage(EntityInterface $entity, string $public_key, string $private_key, string $relay, bool $show_drupal_message) {
    $this->showDrupalMessage = $show_drupal_message;

    // Get content from a field.
    $content = NULL;
    $content_fields = Settings::get('nostr_content_fields', ['body']);
    foreach ($content_fields as $content_field) {
      if ($entity->hasField($content_field)) {
        $content = trim($entity->get($content_field)->value);
        break;
      }
    }

    if (!isset($content)) {
      $this->showDrupalMessage($this->t('No content found to send to the Nostr network.'), 'warning');
      $this->logger->warning('No content found in the fields to send to the Nostr network for @entity_type_id @entity_id', ['@entity_type_id' => $entity->getEntityTypeId(), '@entity_id' => $entity->id()]);
      return;
    }

    $event = [
      'pubkey' => $public_key,
      'created_at' => time(),
      'kind' => 1,
      'tags' => [],
      'content' => $content,
    ];

    $signer = new Sign();
    $event = $signer->signEvent($event, $private_key);
    $message = $signer->generateEvent($event);

    if (!empty($message)) {

      if ($this->currentUser->hasPermission('view nostr debugging')) {
        $this->showDrupalMessage($message, 'warning');
      }

      $client = new Client($relay);
      try {

        $client->send($message);
        $response = json_decode($client->receive());
        $success = TRUE;
        $failure = '';
        if (!empty($response[3])) {
          $success = FALSE;
          $failure = $response[3];
        }

        // View debugging information.
        if ($this->currentUser->hasPermission('view nostr debugging')) {
          $this->showDrupalMessage(print_r($response, 1), 'warning');
        }

        if ($success) {
          $this->showDrupalMessage($this->t('Sent to Nostr network.'));

          // Store the id.
          if (Settings::get('nostr_store_nids', TRUE)) {
            $this->nostrStorage->storeEventId($entity->id(), $entity->getEntityTypeId(), $event['id']);
          }

        }
        else {
          $this->showDrupalMessage($this->t('Error posting to Nostr network: @fail', ['@fail' => $failure]), 'error');
          $this->logger->error('Error posting to Nostr network: @message', ['@message' => $failure]);
        }
      }
      catch (\Exception $e) {
        $this->showDrupalMessage($this->t('Error posting to Nostr network: @message', ['@message' => $e->getMessage()]), 'error');
        $this->logger->error('Error posting to Nostr network: @message', ['@message' => $e->getMessage()]);
      }
    }
    else {
      $this->showDrupalMessage($this->t('Creating the message failed.'), 'error');
      $this->logger->error('Creating the message failed');
    }

  }

  /**
   * Helper function to show a Drupal message.
   *
   * @param string $message
   * @param string $type
   */
  protected function showDrupalMessage(string $message, string $type = 'default') {
    if ($this->showDrupalMessage) {
      switch ($type) {
        case 'error':
          $this->messenger->addError($message);
          break;
        case 'warning':
          $this->messenger->addWarning($message);
          break;
        default:
          $this->messenger->addMessage($message);
          break;
      }
    }
  }

}
